# -*- coding: utf-8 -*-
"""DNA Center Template Programmer API wrapper.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    check_type,
    dict_from_items_with_values,
    apply_path_params,
    dict_of_str,
)


class TemplateProgrammer(object):
    """DNA Center Template Programmer API (version: 1.2.10).

    Wraps the DNA Center Template Programmer
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new TemplateProgrammer object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the DNA Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(TemplateProgrammer, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def gets_the_templates_available(self,
                                     filter_conflicting_templates=None,
                                     product_family=None,
                                     product_series=None,
                                     product_type=None,
                                     project_id=None,
                                     software_type=None,
                                     software_version=None,
                                     headers=None,
                                     payload=None,
                                     active_validation=True,
                                     **request_parameters):
        """List the templates available.

        Args:
            project_id(basestring): projectId query parameter.
            software_type(basestring): softwareType query parameter.
            software_version(basestring): softwareVersion query
                parameter.
            product_family(basestring): productFamily query
                parameter.
            product_series(basestring): productSeries query
                parameter.
            product_type(basestring): productType query parameter.
            filter_conflicting_templates(bool):
                filterConflictingTemplates query
                parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(project_id, basestring)
        check_type(software_type, basestring)
        check_type(software_version, basestring)
        check_type(product_family, basestring)
        check_type(product_series, basestring)
        check_type(product_type, basestring)
        check_type(filter_conflicting_templates, bool)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
            'projectId':
                project_id,
            'softwareType':
                software_type,
            'softwareVersion':
                software_version,
            'productFamily':
                product_family,
            'productSeries':
                product_series,
            'productType':
                product_type,
            'filterConflictingTemplates':
                filter_conflicting_templates,
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_01b09a254b9ab259_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/template')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload, headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_01b09a254b9ab259_v1_2_10', json_data)

    def create_project(self,
                       createTime=None,
                       description=None,
                       id=None,
                       lastUpdateTime=None,
                       name=None,
                       tags=None,
                       templates=None,
                       headers=None,
                       payload=None,
                       active_validation=True,
                       **request_parameters):
        """Creates a new project.

        Args:
            createTime(number): ProjectDTO's createTime.
            description(string): ProjectDTO's description.
            id(string): ProjectDTO's id.
            lastUpdateTime(number): ProjectDTO's lastUpdateTime.
            name(string): ProjectDTO's name.
            tags(list): ProjectDTO's tags (list of string, objects).
            templates: Part of the JSON serializable Python object
                to send in the body of the Request.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
            'createTime':
                createTime,
            'description':
                description,
            'id':
                id,
            'lastUpdateTime':
                lastUpdateTime,
            'name':
                name,
            'tags':
                tags,
            'templates':
                templates,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_00aec9b1422ab27e_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/project')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload)

        return self._object_factory('bpm_00aec9b1422ab27e_v1_2_10', json_data)

    def update_template(self,
                        author=None,
                        composite=None,
                        containingTemplates=None,
                        createTime=None,
                        description=None,
                        deviceTypes=None,
                        failurePolicy=None,
                        id=None,
                        lastUpdateTime=None,
                        name=None,
                        parentTemplateId=None,
                        projectId=None,
                        projectName=None,
                        rollbackTemplateContent=None,
                        rollbackTemplateParams=None,
                        softwareType=None,
                        softwareVariant=None,
                        softwareVersion=None,
                        tags=None,
                        templateContent=None,
                        templateParams=None,
                        version=None,
                        headers=None,
                        payload=None,
                        active_validation=True,
                        **request_parameters):
        """Updates an existing template.

        Args:
            author(string): TemplateDTO's author.
            composite(boolean): TemplateDTO's composite.
            containingTemplates(list): TemplateDTO's
                containingTemplates (list of objects).
            createTime(number): TemplateDTO's createTime.
            description(string): TemplateDTO's description.
            deviceTypes(list): TemplateDTO's deviceTypes (list of
                objects).
            failurePolicy(string): TemplateDTO's failurePolicy.
                Available values are 'ABORT_ON_ERROR',
                'CONTINUE_ON_ERROR',
                'ROLLBACK_ON_ERROR',
                'ROLLBACK_TARGET_ON_ERROR' and
                'ABORT_TARGET_ON_ERROR'.
            id(string): TemplateDTO's id.
            lastUpdateTime(number): TemplateDTO's lastUpdateTime.
            name(string): TemplateDTO's name.
            parentTemplateId(string): TemplateDTO's
                parentTemplateId.
            projectId(string): TemplateDTO's projectId.
            projectName(string): TemplateDTO's projectName.
            rollbackTemplateContent(string): TemplateDTO's
                rollbackTemplateContent.
            rollbackTemplateParams(list): TemplateDTO's
                rollbackTemplateParams (list of
                objects).
            softwareType(string): TemplateDTO's softwareType.
            softwareVariant(string): TemplateDTO's softwareVariant.
            softwareVersion(string): TemplateDTO's softwareVersion.
            tags(list): TemplateDTO's tags (list of string,
                objects).
            templateContent(string): TemplateDTO's templateContent.
            templateParams(list): TemplateDTO's templateParams (list
                of objects).
            version(string): TemplateDTO's version.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
            'author':
                author,
            'composite':
                composite,
            'containingTemplates':
                containingTemplates,
            'createTime':
                createTime,
            'description':
                description,
            'deviceTypes':
                deviceTypes,
            'failurePolicy':
                failurePolicy,
            'id':
                id,
            'lastUpdateTime':
                lastUpdateTime,
            'name':
                name,
            'parentTemplateId':
                parentTemplateId,
            'projectId':
                projectId,
            'projectName':
                projectName,
            'rollbackTemplateContent':
                rollbackTemplateContent,
            'rollbackTemplateParams':
                rollbackTemplateParams,
            'softwareType':
                softwareType,
            'softwareVariant':
                softwareVariant,
            'softwareVersion':
                softwareVersion,
            'tags':
                tags,
            'templateContent':
                templateContent,
            'templateParams':
                templateParams,
            'version':
                version,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_7781fa0548a98342_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/template')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_7781fa0548a98342_v1_2_10', json_data)

    def get_projects(self,
                     name=None,
                     headers=None,
                     payload=None,
                     active_validation=True,
                     **request_parameters):
        """Returns the projects in the system.

        Args:
            name(basestring): Name of project to be searched.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            list: JSON response. A list of MyDict objects.
            Access the object's properties by using the dot notation
            or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(name, basestring)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
            'name':
                name,
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_109d1b4f4289aecd_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/project')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload, headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_109d1b4f4289aecd_v1_2_10', json_data)

    def deploy_template(self,
                        forcePushTemplate=None,
                        isComposite=None,
                        mainTemplateId=None,
                        memberTemplateDeploymentInfo=None,
                        targetInfo=None,
                        templateId=None,
                        headers=None,
                        payload=None,
                        active_validation=True,
                        **request_parameters):
        """Deploys a template.

        Args:
            forcePushTemplate(boolean): TemplateDeploymentInfo's
                forcePushTemplate.
            isComposite(boolean): TemplateDeploymentInfo's
                isComposite.
            mainTemplateId(string): TemplateDeploymentInfo's
                mainTemplateId.
            memberTemplateDeploymentInfo(list):
                TemplateDeploymentInfo's
                memberTemplateDeploymentInfo (list of
                any objects).
            targetInfo(list): TemplateDeploymentInfo's targetInfo
                (list of objects).
            templateId(string): TemplateDeploymentInfo's templateId.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
            'forcePushTemplate':
                forcePushTemplate,
            'isComposite':
                isComposite,
            'mainTemplateId':
                mainTemplateId,
            'memberTemplateDeploymentInfo':
                memberTemplateDeploymentInfo,
            'targetInfo':
                targetInfo,
            'templateId':
                templateId,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_6099da82477b858a_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/template/deploy')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload)

        return self._object_factory('bpm_6099da82477b858a_v1_2_10', json_data)

    def get_template_details(self,
                             template_id,
                             latest_version=None,
                             headers=None,
                             payload=None,
                             active_validation=True,
                             **request_parameters):
        """Returns details of the specified template.

        Args:
            template_id(basestring): templateId path parameter.
            latest_version(bool): latestVersion query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(latest_version, bool)
        check_type(template_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
            'latestVersion':
                latest_version,
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
            'templateId': template_id,
        }

        _payload = {
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_83a3b9404cb88787_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-'
                 + 'programmer/template/${templateId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload, headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_83a3b9404cb88787_v1_2_10', json_data)

    def update_project(self,
                       createTime=None,
                       description=None,
                       id=None,
                       lastUpdateTime=None,
                       name=None,
                       tags=None,
                       templates=None,
                       headers=None,
                       payload=None,
                       active_validation=True,
                       **request_parameters):
        """Updates an existing project.

        Args:
            createTime(number): ProjectDTO's createTime.
            description(string): ProjectDTO's description.
            id(string): ProjectDTO's id.
            lastUpdateTime(number): ProjectDTO's lastUpdateTime.
            name(string): ProjectDTO's name.
            tags(list): ProjectDTO's tags (list of string, objects).
            templates: Part of the JSON serializable Python object
                to send in the body of the Request.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
            'createTime':
                createTime,
            'description':
                description,
            'id':
                id,
            'lastUpdateTime':
                lastUpdateTime,
            'name':
                name,
            'tags':
                tags,
            'templates':
                templates,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_9480fa1f47ca9254_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/project')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_9480fa1f47ca9254_v1_2_10', json_data)

    def get_template_deployment_status(self,
                                       deployment_id,
                                       headers=None,
                                       payload=None,
                                       active_validation=True,
                                       **request_parameters):
        """Returns the status of a deployed template.

        Args:
            deployment_id(basestring): deploymentId path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(deployment_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
            'deploymentId': deployment_id,
        }

        _payload = {
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_9c9a785741cbb41f_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-'
                 + 'programmer/template/deploy/status/${deploymentId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload, headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_9c9a785741cbb41f_v1_2_10', json_data)

    def delete_template(self,
                        template_id,
                        headers=None,
                        payload=None,
                        active_validation=True,
                        **request_parameters):
        """Deletes an existing template.

        Args:
            template_id(basestring): templateId path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(template_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
            'templateId': template_id,
        }

        _payload = {
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_a7b42836408a8e74_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-'
                 + 'programmer/template/${templateId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=params,
                                             json=_payload, headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=params,
                                             json=_payload)

        return self._object_factory('bpm_a7b42836408a8e74_v1_2_10', json_data)

    def version_template(self,
                         comments=None,
                         templateId=None,
                         headers=None,
                         payload=None,
                         active_validation=True,
                         **request_parameters):
        """Creates Versioning for the current contents of the template.

        Args:
            comments(string): TemplateVersionRequestDTO's comments.
            templateId(string): TemplateVersionRequestDTO's
                templateId.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
            'comments':
                comments,
            'templateId':
                templateId,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_62b05b2c40a9b216_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/template/version')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload)

        return self._object_factory('bpm_62b05b2c40a9b216_v1_2_10', json_data)

    def preview_template(self,
                         params=None,
                         templateId=None,
                         headers=None,
                         payload=None,
                         active_validation=True,
                         **request_parameters):
        """Previews an existing template.

        Args:
            params(object): TemplatePreviewRequestDTO's params.
            templateId(string): TemplatePreviewRequestDTO's
                templateId.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
        }

        _payload = {
            'params':
                params,
            'templateId':
                templateId,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_f393abe84989bb48_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-programmer/template/preview')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_f393abe84989bb48_v1_2_10', json_data)

    def delete_project(self,
                       project_id,
                       headers=None,
                       payload=None,
                       active_validation=True,
                       **request_parameters):
        """Deletes an existing Project.

        Args:
            project_id(basestring): projectId path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(project_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
            'projectId': project_id,
        }

        _payload = {
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_d0a1abfa435b841d_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-'
                 + 'programmer/project/${projectId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=params,
                                             json=_payload, headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=params,
                                             json=_payload)

        return self._object_factory('bpm_d0a1abfa435b841d_v1_2_10', json_data)

    def create_template(self,
                        project_id,
                        author=None,
                        composite=None,
                        containingTemplates=None,
                        createTime=None,
                        description=None,
                        deviceTypes=None,
                        failurePolicy=None,
                        id=None,
                        lastUpdateTime=None,
                        name=None,
                        parentTemplateId=None,
                        projectId=None,
                        projectName=None,
                        rollbackTemplateContent=None,
                        rollbackTemplateParams=None,
                        softwareType=None,
                        softwareVariant=None,
                        softwareVersion=None,
                        tags=None,
                        templateContent=None,
                        templateParams=None,
                        version=None,
                        headers=None,
                        payload=None,
                        active_validation=True,
                        **request_parameters):
        """Creates a new template.

        Args:
            author(string): TemplateDTO's author.
            composite(boolean): TemplateDTO's composite.
            containingTemplates(list): TemplateDTO's
                containingTemplates (list of objects).
            createTime(number): TemplateDTO's createTime.
            description(string): TemplateDTO's description.
            deviceTypes(list): TemplateDTO's deviceTypes (list of
                objects).
            failurePolicy(string): TemplateDTO's failurePolicy.
                Available values are 'ABORT_ON_ERROR',
                'CONTINUE_ON_ERROR',
                'ROLLBACK_ON_ERROR',
                'ROLLBACK_TARGET_ON_ERROR' and
                'ABORT_TARGET_ON_ERROR'.
            id(string): TemplateDTO's id.
            lastUpdateTime(number): TemplateDTO's lastUpdateTime.
            name(string): TemplateDTO's name.
            parentTemplateId(string): TemplateDTO's
                parentTemplateId.
            projectId(string): TemplateDTO's projectId.
            projectName(string): TemplateDTO's projectName.
            rollbackTemplateContent(string): TemplateDTO's
                rollbackTemplateContent.
            rollbackTemplateParams(list): TemplateDTO's
                rollbackTemplateParams (list of
                objects).
            softwareType(string): TemplateDTO's softwareType.
            softwareVariant(string): TemplateDTO's softwareVariant.
            softwareVersion(string): TemplateDTO's softwareVersion.
            tags(list): TemplateDTO's tags (list of string,
                objects).
            templateContent(string): TemplateDTO's templateContent.
            templateParams(list): TemplateDTO's templateParams (list
                of objects).
            version(string): TemplateDTO's version.
            project_id(basestring): projectId path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(project_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
            'projectId': project_id,
        }

        _payload = {
            'author':
                author,
            'composite':
                composite,
            'containingTemplates':
                containingTemplates,
            'createTime':
                createTime,
            'description':
                description,
            'deviceTypes':
                deviceTypes,
            'failurePolicy':
                failurePolicy,
            'id':
                id,
            'lastUpdateTime':
                lastUpdateTime,
            'name':
                name,
            'parentTemplateId':
                parentTemplateId,
            'projectId':
                projectId,
            'projectName':
                projectName,
            'rollbackTemplateContent':
                rollbackTemplateContent,
            'rollbackTemplateParams':
                rollbackTemplateParams,
            'softwareType':
                softwareType,
            'softwareVariant':
                softwareVariant,
            'softwareVersion':
                softwareVersion,
            'tags':
                tags,
            'templateContent':
                templateContent,
            'templateParams':
                templateParams,
            'version':
                version,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_f6b119ad4d4aaf16_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-'
                 + 'programmer/project/${projectId}/template')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=params,
                                           json=_payload)

        return self._object_factory('bpm_f6b119ad4d4aaf16_v1_2_10', json_data)

    def get_template_versions(self,
                              template_id,
                              headers=None,
                              payload=None,
                              active_validation=True,
                              **request_parameters):
        """Returns the versions of a specified template.

        Args:
            template_id(basestring): templateId path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            list: JSON response. A list of MyDict objects.
            Access the object's properties by using the dot notation
            or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(template_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        params = {
        }
        params.update(request_parameters)
        params = dict_from_items_with_values(params)

        path_params = {
            'templateId': template_id,
        }

        _payload = {
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_c8bf6b65414a9bc7_v1_2_10')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/template-'
                 + 'programmer/template/version/${templateId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload, headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=params,
                                          json=_payload)

        return self._object_factory('bpm_c8bf6b65414a9bc7_v1_2_10', json_data)



# -*- coding: utf-8 -*-
"""DNA Center Provision NFV data model.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import fastjsonschema
import json
from dnacentersdk.exceptions import MalformedRequest

from builtins import *


class JSONSchemaValidator828828F44F28Bd0D(object):
    """Provision NFV request schema definition."""
    def __init__(self):
        super(JSONSchemaValidator828828F44F28Bd0D, self).__init__()
        self._validator = fastjsonschema.compile(json.loads(
            '''{
                "properties": {
                "provisioning": {
                "description":
                "Provisioning",
                "items": {
                "properties": {
                "device": {
                "description":
                "Device",
                "items": {
                "properties": {
                "customNetworks": {
                "description":
                "Custom Networks",
                "items": {
                "properties": {
                "ipAddressPool": {
                "description":
                "Ip Address Pool",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                "Name",
                "type": [
                "string",
                "null"
                ]
                },
                "port": {
                "description":
                "Port",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "deviceSerialNumber": {
                "description":
                "Device Serial Number",
                "type": [
                "string",
                "null"
                ]
                },
                "ip": {
                "description":
                "Ip",
                "type": [
                "string",
                "null"
                ]
                },
                "serviceProviders": {
                "description":
                "Service Providers",
                "items": {
                "properties": {
                "serviceProvider": {
                "description":
                "Service Provider",
                "type": [
                "string",
                "null"
                ]
                },
                "wanInterface": {
                "description":
                "Wan Interface",
                "properties": {
                "bandwidth": {
                "description":
                "Bandwidth",
                "type": [
                "string",
                "null"
                ]
                },
                "gateway": {
                "description":
                "Gateway",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceName": {
                "description":
                "Interface Name",
                "type": [
                "string",
                "null"
                ]
                },
                "ipAddress": {
                "description":
                "Ip Address",
                "type": [
                "string",
                "null"
                ]
                },
                "subnetmask": {
                "description":
                "Subnetmask",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "services": {
                "description":
                "Services",
                "items": {
                "properties": {
                "adminPasswordHash": {
                "description":
                "Admin Password Hash",
                "type": [
                "string",
                "null"
                ]
                },
                "centralManagerIP": {
                "description":
                "Central Manager IP",
                "type": [
                "string",
                "null"
                ]
                },
                "centralRegistrationKey": {
                "description":
                "Central Registration Key",
                "type": [
                "string",
                "null"
                ]
                },
                "commonKey": {
                "description":
                "Common Key",
                "type": [
                "string",
                "null"
                ]
                },
                "mode": {
                "description":
                "Mode",
                "type": [
                "string",
                "null"
                ]
                },
                "systemIp": {
                "description":
                "System Ip",
                "type": [
                "string",
                "null"
                ]
                },
                "type": {
                "description":
                "Type",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "subPools": {
                "description":
                "Sub Pools",
                "items": {
                "properties": {
                "gateway": {
                "description":
                "Gateway",
                "type": [
                "string",
                "null"
                ]
                },
                "ipSubnet": {
                "description":
                "Ip Subnet",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                "Name",
                "type": [
                "string",
                "null"
                ]
                },
                "parentPoolName": {
                "description":
                "Parent Pool Name",
                "type": [
                "string",
                "null"
                ]
                },
                "type": {
                "description":
                "Type",
                "enum": [
                "Lan",
                "Management",
                "Service",
                "Wan",
                "Generic",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "tagName": {
                "description":
                "Tag Name",
                "type": [
                "string",
                "null"
                ]
                },
                "templateParam": {
                "description":
                "Template Param",
                "properties": {
                "asav": {
                "description":
                "Asav",
                "properties": {
                "var1": {
                "description":
                "Var1",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "nfvis": {
                "description":
                "Nfvis",
                "properties": {
                "var1": {
                "description":
                "Var1",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "vlan": {
                "description":
                "Vlan",
                "items": {
                "properties": {
                "id": {
                "description":
                "Id",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaces": {
                "description":
                "Interfaces",
                "type": [
                "string",
                "null"
                ]
                },
                "network": {
                "description":
                "Network",
                "type": [
                "string",
                "null"
                ]
                },
                "type": {
                "description":
                "Type",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "site": {
                "description":
                "Site",
                "properties": {
                "area": {
                "description":
                "Area",
                "properties": {
                "name": {
                "description":
                "Name",
                "type": [
                "string",
                "null"
                ]
                },
                "parentName": {
                "description":
                "Parent Name",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "building": {
                "description":
                "Building",
                "properties": {
                "address": {
                "description":
                "Address",
                "type": [
                "string",
                "null"
                ]
                },
                "latitude": {
                "type": [
                "number",
                "null"
                ]
                },
                "longitude": {
                "type": [
                "number",
                "null"
                ]
                },
                "name": {
                "description":
                "Name",
                "type": [
                "string",
                "null"
                ]
                },
                "parentName": {
                "description":
                "Parent Name",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "floor": {
                "description":
                "Floor",
                "properties": {
                "height": {
                "type": [
                "number",
                "null"
                ]
                },
                "length": {
                "type": [
                "number",
                "null"
                ]
                },
                "name": {
                "description":
                "Name",
                "type": [
                "string",
                "null"
                ]
                },
                "parentName": {
                "description":
                "Parent Name",
                "type": [
                "string",
                "null"
                ]
                },
                "rfModel": {
                "description":
                "Rf Model",
                "type": [
                "string",
                "null"
                ]
                },
                "width": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "siteProfileName": {
                "description":
                "Site Profile Name",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "siteProfile": {
                "description":
                "Site Profile",
                "items": {
                "properties": {
                "device": {
                "description":
                "Device",
                "items": {
                "properties": {
                "customNetworks": {
                "description":
                "Custom Networks",
                "items": {
                "properties": {
                "connectionType": {
                "description":
                "Connection Type",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                "Name",
                "type": [
                "string",
                "null"
                ]
                },
                "networkMode": {
                "description":
                "Network Mode",
                "type": [
                "string",
                "null"
                ]
                },
                "servicesToConnect": {
                "description":
                "Services To Connect",
                "items": {
                "properties": {
                "service": {
                "description":
                "Service",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "vlan": {
                "description":
                "Vlan",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "customServices": {
                "description":
                "Custom Services",
                "items": {
                "properties": {
                "applicationType": {
                "description":
                "Application Type",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                "Name",
                "type": [
                "string",
                "null"
                ]
                },
                "profile": {
                "description":
                "Profile",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "customTemplate": {
                "description":
                "Custom Template",
                "items": {
                "properties": {
                "deviceType": {
                "description":
                "Device Type",
                "enum": [
                "NFVIS",
                "ASAv",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "template": {
                "description":
                "Template",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "deviceType": {
                "description":
                "Device Type",
                "enum": [
                "ENCS5100",
                "ENCS5400",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "dia": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "serviceProviders": {
                "description":
                "Service Providers",
                "items": {
                "properties": {
                "connect": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "defaultGateway": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "linkType": {
                "description":
                "Link Type",
                "enum": [
                "GigabitEthernet",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "serviceProvider": {
                "description":
                "Service Provider",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "services": {
                "description":
                "Services",
                "items": {
                "properties": {
                "mode": {
                "description":
                "Mode",
                "type": [
                "string",
                "null"
                ]
                },
                "profile": {
                "description":
                "Profile",
                "type": [
                "string",
                "null"
                ]
                },
                "type": {
                "description":
                "Type",
                "enum": [
                "isr",
                "vedge",
                "waas",
                "asa",
                "ngfw",
                "paloalto",
                "fortinet",
                "checkpoint",
                "riverbed",
                "silverpeak",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "tagName": {
                "description":
                "Tag Name",
                "type": [
                "string",
                "null"
                ]
                },
                "vlan": {
                "description":
                "Vlan",
                "items": {
                "properties": {
                "id": {
                "description":
                "Id",
                "type": [
                "string",
                "null"
                ]
                },
                "type": {
                "description":
                "Type",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "siteProfileName": {
                "description":
                "Site Profile Name",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": "object"
                }'''.replace("\n" + ' ' * 16, '')
        ))

    def validate(self, request):
        try:
            self._validator(request)
        except fastjsonschema.exceptions.JsonSchemaException as e:
            raise MalformedRequest(
                '{} is invalid. Reason: {}'.format(request, e.message)
            )



# -*- coding: utf-8 -*-
"""DNA Center Get Discoveries by range data model.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import fastjsonschema
import json
from dnacentersdk.exceptions import MalformedRequest

from builtins import *


class JSONSchemaValidator33B799D04D0A8907(object):
    """Get Discoveries by range request schema definition."""
    def __init__(self):
        super(JSONSchemaValidator33B799D04D0A8907, self).__init__()
        self._validator = fastjsonschema.compile(json.loads(
            '''{
                "properties": {
                "response": {
                "description":
                 "",
                "items": {
                "properties": {
                "attributeInfo": {
                "description":
                 "",
                "properties": {},
                "type": [
                "object",
                "null"
                ]
                },
                "cdpLevel": {
                "type": [
                "number",
                "null"
                ]
                },
                "deviceIds": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "discoveryCondition": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "discoveryStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "discoveryType": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "enablePasswordList": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "globalCredentialIdList": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null",
                "object"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "httpReadCredential": {
                "description":
                 "",
                "properties": {
                "comments": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "credentialType": {
                "description":
                 "",
                "enum": [
                "GLOBAL",
                "APP",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "description":
                 {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "instanceTenantId": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "instanceUuid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "password": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "port": {
                "type": [
                "number",
                "null"
                ]
                },
                "secure": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "username": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "httpWriteCredential": {
                "description":
                 "",
                "properties": {
                "comments": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "credentialType": {
                "description":
                 "",
                "enum": [
                "GLOBAL",
                "APP",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "description":
                 {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "instanceTenantId": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "instanceUuid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "password": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "port": {
                "type": [
                "number",
                "null"
                ]
                },
                "secure": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "username": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ipAddressList": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ipFilterList": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "isAutoCdp": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "lldpLevel": {
                "type": [
                "number",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "netconfPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "numDevices": {
                "type": [
                "number",
                "null"
                ]
                },
                "parentDiscoveryId": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "passwordList": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "preferredMgmtIPMethod": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocolOrder": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "retryCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "snmpAuthPassphrase": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpAuthProtocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpMode": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpPrivPassphrase": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpPrivProtocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpRoCommunity": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpRoCommunityDesc": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpRwCommunity": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpRwCommunityDesc": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "snmpUserName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "timeOut": {
                "type": [
                "number",
                "null"
                ]
                },
                "updateMgmtIp": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "userNameList": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null",
                "object"
                ]
                },
                "version": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": "object"
                }'''.replace("\n" + ' ' * 16, '')
        ))

    def validate(self, request):
        try:
            self._validator(request)
        except fastjsonschema.exceptions.JsonSchemaException as e:
            raise MalformedRequest(
                '{} is invalid. Reason: {}'.format(request, e.message)
            )



# -*- coding: utf-8 -*-
"""DNA Center Response Codes.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)


RESPONSE_CODES = {
    200: "Successful request with body content.",
    201: "Fulfilled request, a new resource has been created.",
    202: "The request was accepted for processing, but the processing has not"
         " been completed.",
    204: "Successful request without body content.",
    206: "The request included a Range Header, and the server responded with "
         "the partial content matching the range.",
    400: "The request was invalid or cannot be otherwise served.",
    401: "Authentication credentials were missing or incorrect.",
    403: "The request is understood, but it has been refused or access is not "
         "allowed.",
    404: "The URI requested is invalid or the resource requested, such as a "
         "user, does not exist. Also returned when the requested format is "
         "not supported by the requested method.",
    409: "The request could not be processed because it conflicts with some "
         "established rule of the system.",
    415: "The request was made to a resource without specifying a media type "
         "or used a media type that is not supported.",
    500: "The server could not fulfill the request.",
    501: "The server has not implemented the functionality required to fulfill"
         " the request.",
    503: "Server is overloaded with requests. Try again later.",
    504: "The server did not respond inside time restrictions and timed-out.",
    # Other http statuses
    405: "The request was made to a resource using an HTTP request method "
         "that is not supported.",
    429: "Too many requests have been sent in a given amount of time and the "
         "request has been rate limited. A Retry-After header should be "
         "present that specifies how many seconds you need to wait before a "
         "successful request can be made.",
    502: "The server received an invalid response from an upstream server "
         "while processing the request. Try again later."
}

RATE_LIMIT_RESPONSE_CODE = 429

EXPECTED_RESPONSE_CODE = {
    'GET': [200, 202, 204, 206],
    'POST': [200, 201, 202, 204, 206],
    'PUT': [200, 201, 202, 204, 206],
    'DELETE': [200, 202, 204, 206],
}



# -*- coding: utf-8 -*-
"""dnacentersdk setup module."""


import os
from codecs import open

from setuptools import find_packages, setup


__copyright__ = "Copyright (c) 2019 Cisco and/or its affiliates."
__license__ = "MIT"


PACKAGE_NAME = 'dnacentersdk'

PACKAGE_KEYWORDS = [
    'cisco',
    'dna',
    'dnacenter',
    'python',
    'api',
    'sdk',
]

PACKAGE_CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Intended Audience :: Information Technology',
    'Intended Audience :: System Administrators',
    'Intended Audience :: Telecommunications Industry',
    'Intended Audience :: Education',
    'Natural Language :: English',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Topic :: System',
    'Topic :: System :: Networking',
    'Topic :: Utilities'
]

INSTALLATION_REQUIREMENTS = [
    'future',
    'requests>=2.4.2',
    'fastjsonschema>=2.12',
    'requests-toolbelt>=0.9.1',
]


project_root = os.path.abspath(os.path.dirname(__file__))


# Get package metadata
metadata = {}
with open(os.path.join(project_root, PACKAGE_NAME, '_metadata.py')) as f:
    exec(f.read(), metadata)


# Get the long description from the project's README.rst file
with open(os.path.join(project_root, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name=PACKAGE_NAME,

    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    description=metadata['__description__'],
    long_description=long_description,

    url=metadata['__url__'],
    download_url=metadata['__download_url__'],

    author=metadata['__author__'],
    author_email=metadata['__author_email__'],

    license=metadata['__license__'] + '; ' + metadata['__copyright__'],

    classifiers=PACKAGE_CLASSIFIERS,

    keywords=" ".join(PACKAGE_KEYWORDS),

    packages=find_packages(include=[PACKAGE_NAME, PACKAGE_NAME + '.*']),

    install_requires=INSTALLATION_REQUIREMENTS,
)



# -*- coding: utf-8 -*-
"""DNA Center Retrieves previous Pathtrace data model.

Copyright (c) 2019 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import fastjsonschema
import json
from dnacentersdk.exceptions import MalformedRequest

from builtins import *


class JSONSchemaValidator7Ab9A8Bd4F3B86A4(object):
    """Retrieves previous Pathtrace request schema definition."""
    def __init__(self):
        super(JSONSchemaValidator7Ab9A8Bd4F3B86A4, self).__init__()
        self._validator = fastjsonschema.compile(json.loads(
            '''{
                "properties": {
                "response": {
                "description":
                 "",
                "properties": {
                "detailedStatus": {
                "description":
                 "",
                "properties": {
                "aclTraceCalculation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "aclTraceCalculationFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "lastUpdate": {
                "description":
                 "",
                "type": [
                "string",
                "null",
                "number"
                ]
                },
                "networkElements": {
                "description":
                 "",
                "items": {
                "properties": {
                "accuracyList": {
                "description":
                 "",
                "items": {
                "properties": {
                "percent": {
                "type": [
                "number",
                "null"
                ]
                },
                "reason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "detailedStatus": {
                "description":
                 "",
                "properties": {
                "aclTraceCalculation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "aclTraceCalculationFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "deviceStatistics": {
                "description":
                 "",
                "properties": {
                "cpuStatistics": {
                "description":
                 "",
                "properties": {
                "fiveMinUsageInPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "fiveSecsUsageInPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "oneMinUsageInPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "memoryStatistics": {
                "description":
                 "",
                "properties": {
                "memoryUsage": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                },
                "totalMemory": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "deviceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "deviceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "egressPhysicalInterface": {
                "description":
                 "",
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "egressVirtualInterface": {
                "description":
                 "",
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "flexConnect": {
                "description":
                 "",
                "properties": {
                "authentication": {
                "description":
                 "",
                "enum": [
                "LOCAL",
                "CENTRAL",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "dataSwitching": {
                "description":
                 "",
                "enum": [
                "LOCAL",
                "CENTRAL",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "egressAclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "ingressAclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "wirelessLanControllerId": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "wirelessLanControllerName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ingressPhysicalInterface": {
                "description":
                 "",
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "ingressVirtualInterface": {
                "description":
                 "",
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "ip": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "linkInformationSource": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "perfMonCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "perfMonCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "perfMonStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "byteRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "destIpAddress": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputInterface": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ipv4DSCP": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ipv4TTL": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputInterface": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "packetBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "packetCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "packetLoss": {
                "type": [
                "number",
                "null"
                ]
                },
                "packetLossPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                },
                "rtpJitterMax": {
                "type": [
                "number",
                "null"
                ]
                },
                "rtpJitterMean": {
                "type": [
                "number",
                "null"
                ]
                },
                "rtpJitterMin": {
                "type": [
                "number",
                "null"
                ]
                },
                "sourceIpAddress": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "role": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ssid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "tunnels": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "type": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "wlanId": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "networkElementsInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "accuracyList": {
                "description":
                 "",
                "items": {
                "properties": {
                "percent": {
                "type": [
                "number",
                "null"
                ]
                },
                "reason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "detailedStatus": {
                "description":
                 "",
                "properties": {
                "aclTraceCalculation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "aclTraceCalculationFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "deviceStatistics": {
                "description":
                 "",
                "properties": {
                "cpuStatistics": {
                "description":
                 "",
                "properties": {
                "fiveMinUsageInPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "fiveSecsUsageInPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "oneMinUsageInPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "memoryStatistics": {
                "description":
                 "",
                "properties": {
                "memoryUsage": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                },
                "totalMemory": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "deviceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "deviceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "egressInterface": {
                "description":
                 "",
                "properties": {
                "physicalInterface": {
                "description":
                 "",
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "virtualInterface": {
                "description":
                 "",
                "items": {
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "flexConnect": {
                "description":
                 "",
                "properties": {
                "authentication": {
                "description":
                 "",
                "enum": [
                "LOCAL",
                "CENTRAL",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "dataSwitching": {
                "description":
                 "",
                "enum": [
                "LOCAL",
                "CENTRAL",
                null
                ],
                "type": [
                "string",
                "null"
                ]
                },
                "egressAclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "ingressAclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "wirelessLanControllerId": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "wirelessLanControllerName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ingressInterface": {
                "description":
                 "",
                "properties": {
                "physicalInterface": {
                "description":
                 "",
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "virtualInterface": {
                "description":
                 "",
                "items": {
                "properties": {
                "aclAnalysis": {
                "description":
                 "",
                "properties": {
                "aclName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingAces": {
                "description":
                 "",
                "items": {
                "properties": {
                "ace": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "matchingPorts": {
                "description":
                 "",
                "items": {
                "properties": {
                "ports": {
                "description":
                 "",
                "items": {
                "properties": {
                "destPorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "sourcePorts": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "result": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatistics": {
                "description":
                 "",
                "properties": {
                "adminStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueFlushes": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputQueueMaxDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "inputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "operationalStatus": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "outputDrop": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputQueueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputRatebps": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "interfaceStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "interfaceStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "pathOverlayInfo": {
                "description":
                 "",
                "items": {
                "properties": {
                "controlPlane": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dataPacketEncapsulation": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vxlanInfo": {
                "description":
                 "",
                "properties": {
                "dscp": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vnid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "classMapName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "dropRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "numBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "numPackets": {
                "type": [
                "number",
                "null"
                ]
                },
                "offeredRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueBandwidthbps": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "queueDepth": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueNoBufferDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "queueTotalDrops": {
                "type": [
                "number",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "qosStatsCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "qosStatsCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "usedVlan": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "vrfName": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "ip": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "linkInformationSource": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "name": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "perfMonCollection": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "perfMonCollectionFailureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "perfMonitorStatistics": {
                "description":
                 "",
                "items": {
                "properties": {
                "byteRate": {
                "type": [
                "number",
                "null"
                ]
                },
                "destIpAddress": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inputInterface": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ipv4DSCP": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ipv4TTL": {
                "type": [
                "number",
                "null"
                ]
                },
                "outputInterface": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "packetBytes": {
                "type": [
                "number",
                "null"
                ]
                },
                "packetCount": {
                "type": [
                "number",
                "null"
                ]
                },
                "packetLoss": {
                "type": [
                "number",
                "null"
                ]
                },
                "packetLossPercentage": {
                "type": [
                "number",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "refreshedAt": {
                "type": [
                "number",
                "null"
                ]
                },
                "rtpJitterMax": {
                "type": [
                "number",
                "null"
                ]
                },
                "rtpJitterMean": {
                "type": [
                "number",
                "null"
                ]
                },
                "rtpJitterMin": {
                "type": [
                "number",
                "null"
                ]
                },
                "sourceIpAddress": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "role": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "ssid": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "tunnels": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "type": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "wlanId": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "properties": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "request": {
                "description":
                 "",
                "properties": {
                "controlPath": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "createTime": {
                "type": [
                "number",
                "null"
                ]
                },
                "destIP": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "destPort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "failureReason": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "id": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "inclusions": {
                "description":
                 "",
                "items": {
                "type": [
                "string",
                "null"
                ]
                },
                "type": [
                "array",
                "null"
                ]
                },
                "lastUpdateTime": {
                "type": [
                "number",
                "null"
                ]
                },
                "periodicRefresh": {
                "type": [
                "boolean",
                "null"
                ]
                },
                "protocol": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourceIP": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "sourcePort": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                },
                "status": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                }
                },
                "type": [
                "object",
                "null"
                ]
                },
                "version": {
                "description":
                 "",
                "type": [
                "string",
                "null"
                ]
                }
                },
                "type": "object"
                }'''.replace("\n" + ' ' * 16, '')
        ))

    def validate(self, request):
        try:
            self._validator(request)
        except fastjsonschema.exceptions.JsonSchemaException as e:
            raise MalformedRequest(
                '{} is invalid. Reason: {}'.format(request, e.message)
            )
